@@ -1,5 +1,5 @@
 /* nclua.canvas -- The NCLua Canvas module.
-   Copyright (C) 2013-2014 PUC-Rio/Laboratorio TeleMidia
+   Copyright (C) 2013-2015 PUC-Rio/Laboratorio TeleMidia
 
 This file is part of NCLua.
 
@@ -28,7 +28,7 @@ along with NCLua.  If not, see <http://www.gnu.org/licenses/>.  */
 #include <glib-object.h>
 #include <pango/pangocairo.h>
 
-#if HAVE_GTK
+#if defined HAVE_GTK && HAVE_GTK
 # include <gtk/gtk.h>
 #endif
 
@@ -71,7 +71,7 @@ canvas_check (lua_State *L, int index, cairo_t **cr)
 {
   canvas_t *canvas;
   canvas = (canvas_t *) luaL_checkudata (L, index, CANVAS);
-  test_and_set (cr != NULL, *cr, canvas->cr);
+  set_if_nonnull (cr, canvas->cr);
   return canvas;
 }
 
@@ -148,7 +148,7 @@ cairox_surface_create_from_file (const char *path, cairo_surface_t **dup)
 {
   cairo_surface_t *sfc;
 
-#ifdef HAVE_GTK
+#if defined HAVE_GTK && HAVE_GTK
 
   GdkPixbuf *pixbuf;
   GError *error = NULL;
@@ -168,10 +168,12 @@ cairox_surface_create_from_file (const char *path, cairo_surface_t **dup)
   w = gdk_pixbuf_get_width (pixbuf);
   h = gdk_pixbuf_get_height (pixbuf);
   sfc = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, w, h);
+  assert (sfc != NULL);         /* cannot fail */
   if (unlikely (!cairox_surface_is_valid (sfc)))
     return cairo_surface_status (sfc);
 
   cr = cairo_create (sfc);
+  assert (cr != NULL);          /* cannot fail */
   if (!cairox_is_valid (cr))
     return cairo_status (cr);
 
@@ -185,6 +187,7 @@ cairox_surface_create_from_file (const char *path, cairo_surface_t **dup)
 #else
 
   sfc = cairo_image_surface_create_from_png (path);
+  assert (sfc != NULL);         /* cannot fail */
   if (unlikely (!cairox_surface_is_valid (sfc)))
     return cairo_surface_status (sfc);
 
@@ -219,7 +222,8 @@ cairox_surface_get_rotation_bounding_box (cairo_surface_t *src,
    CAIRO_STATUS_SUCCESS if successful, or an error status otherwise.  */
 
 static cairo_status_t
-cairox_surface_rotate_and_flip (cairo_surface_t *src, cairo_surface_t **dup,
+cairox_surface_rotate_and_flip (cairo_surface_t *src,
+                                cairo_surface_t **dup,
                                 double r, int fx, int fy)
 {
   cairo_surface_t *sfc;
@@ -230,10 +234,12 @@ cairox_surface_rotate_and_flip (cairo_surface_t *src, cairo_surface_t **dup,
   cairox_surface_get_rotation_bounding_box (src, &w, &h, r);
 
   sfc = cairo_surface_create_similar (src, CAIRO_CONTENT_COLOR_ALPHA, w, h);
+  assert (sfc != NULL);         /* cannot fail */
   if (unlikely (!cairox_surface_is_valid (sfc)))
     return cairo_surface_status (sfc);
 
   cr = cairo_create (sfc);
+  assert (cr != NULL);          /* cannot fail */
   if (unlikely (!cairox_is_valid (cr)))
     {
       cairo_surface_destroy (sfc);
@@ -282,10 +288,12 @@ cairox_surface_duplicate (cairo_surface_t *src, cairo_surface_t **dup,
 
   sfc = cairo_surface_create_similar (src, CAIRO_CONTENT_COLOR_ALPHA,
                                       crop->width, crop->height);
+  assert (sfc != NULL);         /* cannot fail */
   if (unlikely (!cairox_surface_is_valid (sfc)))
     return cairo_surface_status (sfc);
 
   cr = cairo_create (sfc);
+  assert (cr != NULL);          /* cannot fail */
   if (unlikely (!cairox_is_valid (cr)))
     {
       cairo_surface_destroy (sfc);
@@ -364,6 +372,7 @@ l_canvas_new (lua_State *L)
       int w = luaL_checkint (L, 2);
       int h = luaL_checkint (L, 3);
       sfc = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, w, h);
+      assert (sfc != NULL);     /* cannot fail */
       if (unlikely (!cairox_surface_is_valid (sfc)))
         {
           lua_pushnil (L);
@@ -373,6 +382,7 @@ l_canvas_new (lua_State *L)
     }
 
   cr = cairo_create (sfc);
+  assert (cr != NULL);          /* cannot fail */
   if (unlikely (!cairox_is_valid (cr)))
     return error_throw_invalid_cr (L, cr);
 
@@ -397,6 +407,7 @@ l_canvas_new (lua_State *L)
     }
 
   canvas = (canvas_t *) lua_newuserdata (L, sizeof (*canvas));
+  assert (canvas != NULL);      /* cannot fail */
   canvas->sfc = sfc;
   canvas->back_sfc = back_sfc;
   canvas->cr = cr;
@@ -475,9 +486,9 @@ _l_canvas_dump_to_file (lua_State *L)
  *
  * The following FORMAT strings are supported:
  *
- *   ARGB32 - each pixel is 32-bit quantity, with alpha in the upper 8 bits,
- *            then red, then green, then blue; the 32-bit quantities are
- *            stored native-endian; pre-multiplied alpha is used;
+ *   ARGB32 - each pixel is a 32-bit quantity, with alpha in the upper 8
+ *            bits, then red, then green, then blue; the 32-bit quantities
+ *            are stored native-endian; pre-multiplied alpha is used;
  *
  *   RGB24  - each pixel is a 32-bit quantity, with the upper 8 bits unused;
  *            red, green, and blue are stored in the remaining 24 bits in
@@ -506,10 +517,12 @@ _l_canvas_dump_to_memory (lua_State *L)
 
   aux_sfc = cairo_image_surface_create_for_data (buf, (cairo_format_t) fmt,
                                                  w, h, s);
+  assert (aux_sfc != NULL);     /* cannot fail */
   if (unlikely (!cairox_surface_is_valid (aux_sfc)))
     return error_throw_invalid_surface (L, aux_sfc);
 
   aux_cr = cairo_create (aux_sfc);
+  assert (aux_cr != NULL);      /* cannot fail */
   if (unlikely (!cairox_is_valid (aux_cr)))
     {
       cairo_surface_destroy (aux_sfc);
@@ -743,6 +756,7 @@ l_canvas_attrCrop (lua_State *L)
       rect.width = canvas->width;
       rect.height = canvas->height;
       region = cairo_region_create_rectangle (&rect);
+      assert (region != NULL);  /* cannot fail */
       if (unlikely (!cairox_region_is_valid (region)))
         return error_throw (L, cairox_region_status_desc (region));
 
@@ -928,8 +942,7 @@ l_canvas_attrFont (lua_State *L)
         }
 
       font = pango_font_description_new ();
-      assert (font != NULL);
-
+      assert (font != NULL);    /* cannot fail */
       pango_font_description_set_family (font, family);
       pango_font_description_set_size (font, (int) (size * PANGO_SCALE));
       pango_font_description_set_weight (font, (PangoWeight) weight);
@@ -992,7 +1005,7 @@ l_canvas_attrOpacity (lua_State *L)
     {
       int opacity;
       opacity = luaL_checkint (L, 2);
-      canvas->opacity = (unsigned char) (range (opacity, 0, 255));
+      canvas->opacity = (unsigned char) (clamp (opacity, 0, 255));
       return 0;
     }
 }
@@ -1103,7 +1116,7 @@ l_canvas_clear (lua_State *L)
   x = luaL_optnumber (L, 2, 0);
   y = luaL_optnumber (L, 3, 0);
   w = luaL_optnumber (L, 4, canvas->width);
-  h = luaL_optnumber (L, 5, canvas->width);
+  h = luaL_optnumber (L, 5, canvas->height);
 
   cairo_save (cr);
   cairo_reset_clip (cr);
@@ -1143,10 +1156,10 @@ l_canvas_compose (lua_State *L)
 
   src = canvas_check (L, 4, NULL);
   luaL_argcheck (L, dest != src, 4, "cannot compose onto itself");
-  src_x = range (luaL_optint (L, 5, src->crop.x), 0, src->width);
-  src_y = range (luaL_optint (L, 6, src->crop.y), 0, src->height);
-  src_w = range (luaL_optint (L, 7, src->crop.width), 0, src->width);
-  src_h = range (luaL_optint (L, 8, src->crop.height), 0, src->height);
+  src_x = clamp (luaL_optint (L, 5, src->crop.x), 0, src->width);
+  src_y = clamp (luaL_optint (L, 6, src->crop.y), 0, src->height);
+  src_w = clamp (luaL_optint (L, 7, src->crop.width), 0, src->width);
+  src_h = clamp (luaL_optint (L, 8, src->crop.height), 0, src->height);
 
   if (src_x > 0 || src_y > 0 || src_w < src->width || src_h < src->height)
     {
@@ -1394,7 +1407,7 @@ l_canvas_drawRoundRect (lua_State *L)
   y = luaL_checknumber (L, 4);
   w = luaL_checknumber (L, 5);
   h = luaL_checknumber (L, 6);
-  r = range (luaL_checknumber (L, 7), 0, min (w, h) / 2);
+  r = clamp (luaL_checknumber (L, 7), 0, min (w, h) / 2);
 
   cairo_save (cr);
   cairo_arc (cr, x + r, y + r, r, M_PI, 1.5 * M_PI);
@@ -1440,7 +1453,7 @@ l_canvas_drawText (lua_State *L)
   text = luaL_checkstring (L, 5);
 
   layout = pango_cairo_create_layout (cr);
-  assert (layout != NULL);
+  assert (layout != NULL);      /* cannot fail */
   pango_layout_set_text (layout, text, -1);
   pango_layout_set_font_description (layout, canvas->font);
 
@@ -1471,6 +1484,7 @@ l_canvas_flush (lua_State *L)
     return 0;                   /* nothing to do */
 
   aux_cr = cairo_create (canvas->sfc);
+  assert (aux_cr != NULL);      /* cannot fail */
   if (unlikely (!cairox_is_valid (aux_cr)))
     return error_throw_invalid_cr (L, aux_cr);
 
@@ -1501,7 +1515,7 @@ l_canvas_measureText (lua_State *L)
   text = luaL_checkstring (L, 2);
 
   layout = pango_cairo_create_layout (cr);
-  assert (layout != NULL);
+  assert (layout != NULL);      /* cannot fail */
   pango_layout_set_text (layout, text, -1);
   pango_layout_set_font_description (layout, canvas->font);
   pango_layout_get_size (layout, &w, &h);
@@ -1610,6 +1624,7 @@ l_canvas_pixel (lua_State *L)
       a = luaL_optnumber (L, 7, 255);
 
       aux_cr = cairo_create (canvas->sfc);
+      assert (aux_cr != NULL);  /* cannot fail */
       if (unlikely (!cairox_is_valid (aux_cr)))
         return error_throw_invalid_cr (L, aux_cr);
 
