@@ -1,5 +1,5 @@
 /* nclua.c -- A GTK+-3.0 standalone NCLua player.
-   Copyright (C) 2013-2014 PUC-Rio/Laboratorio TeleMidia
+   Copyright (C) 2013-2015 PUC-Rio/Laboratorio TeleMidia
 
 This file is part of NCLua.
 
@@ -19,6 +19,7 @@ along with NCLua.  If not, see <http://www.gnu.org/licenses/>.  */
 #include <config.h>
 #include <assert.h>
 #include <stdlib.h>
+#include <setjmp.h>
 
 #include <lua.h>
 #include <lualib.h>
@@ -36,6 +37,15 @@ along with NCLua.  If not, see <http://www.gnu.org/licenses/>.  */
 #define HEIGHT 600
 static ncluaw_t *ncluaw_state;
 static GtkWidget *app;
+static jmp_buf panic_jmp;
+
+static void
+panic (arg_unused (ncluaw_t *nw), const char *message)
+{
+  luax_dump_stack (cast (lua_State *, ncluaw_debug_get_lua_state (nw)));
+  fprintf (stderr, "error: %s\n", message);
+  longjmp (panic_jmp, 1);
+}
 
 #if GTK_CHECK_VERSION(3,8,0)
 static gboolean
@@ -133,17 +143,48 @@ keyboard_callback (arg_unused (GtkWidget *widget), GdkEventKey *e,
     }
 
   ncluaw_send_key_event (ncluaw_state, (const char *) type, key);
-  if (free_key)
+  if (free_key)                 /* do not remove */
     g_free (deconst (char *, key));
 
   return TRUE;
 }
 
+static gboolean
+pointer_motion_callback (arg_unused (GtkWidget *widget),
+                         GdkEventMotion *e, arg_unused (const char *type))
+{
+  ncluaw_send_pointer_event (ncluaw_state, "move", (int) e->x, (int) e->y);
+  return TRUE;
+}
+
+static gboolean
+pointer_click_callback (arg_unused (GtkWidget *widget), GdkEventButton *e,
+                        arg_unused (gpointer data))
+{
+  const char *type;
+
+  switch (e->type)
+    {
+    case GDK_BUTTON_PRESS:
+      type = "press";
+      break;
+    case GDK_BUTTON_RELEASE:
+      type = "release";
+      break;
+    default:
+      return TRUE;              /* nothing to do */
+    }
+
+  ncluaw_send_pointer_event (ncluaw_state, type, (int) e->x, (int) e->y);
+  return TRUE;
+}
+
 int
 main (int argc, char **argv)
 {
   GtkWidget *canvas;
   char *errmsg = NULL;
+  volatile int status = EXIT_SUCCESS;
 
   gchar *dirname;
   gchar *basename;
@@ -168,10 +209,13 @@ main (int argc, char **argv)
       exit (EXIT_FAILURE);
     }
 
+  ncluaw_at_panic (ncluaw_state, panic);
+
   g_free (dirname);
   g_free (basename);
 
   app = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  assert (app != NULL);         /* cannot fail */
   gtk_window_set_title (GTK_WINDOW (app), "NCLua");
   gtk_widget_set_size_request (app, WIDTH, HEIGHT);
   gtk_window_set_resizable (GTK_WINDOW (app), FALSE);
@@ -187,22 +231,45 @@ main (int argc, char **argv)
                     deconst (void *, "release"));
 
   canvas = gtk_drawing_area_new ();
+  assert (canvas != NULL);      /* cannot fail */
+  gtk_widget_add_events (canvas, GDK_BUTTON_PRESS_MASK
+                         | GDK_BUTTON_RELEASE_MASK
+                         | GDK_POINTER_MOTION_MASK);
+
+  g_signal_connect (canvas, "button-press-event",
+                    G_CALLBACK (pointer_click_callback), NULL);
+
+  g_signal_connect (canvas, "button-release-event",
+                    G_CALLBACK (pointer_click_callback), NULL);
+
+  g_signal_connect (canvas, "motion-notify-event",
+                    G_CALLBACK (pointer_motion_callback), NULL);
+
   g_signal_connect (canvas, "draw", G_CALLBACK (draw_callback), NULL);
+
   gtk_container_add (GTK_CONTAINER (app), canvas);
 
 #if GTK_CHECK_VERSION(3,8,0)
   gtk_widget_add_tick_callback (canvas, (GtkTickCallback) cycle_callback,
                                 NULL, NULL);
 #else
-  g_timeout_add (1000/60, (GSourceFunc) cycle_callback, canvas);
+  g_timeout_add (1000 / 60, (GSourceFunc) cycle_callback, canvas);
 #endif
 
   /* NCL */
   ncluaw_send_ncl_event (ncluaw_state, "presentation", "start", "", NULL);
 
   gtk_widget_show_all (app);
+
+  if (setjmp (panic_jmp))
+    {
+      status = EXIT_FAILURE;
+      goto panic;
+    }
+
   gtk_main ();
 
+ panic:
   ncluaw_close (ncluaw_state);
-  exit (EXIT_SUCCESS);
+  exit (status);
 }
