@@ -1,5 +1,5 @@
 /* tests.h -- Common declarations for tests.
-   Copyright (C) 2013-2014 PUC-Rio/Laboratorio TeleMidia
+   Copyright (C) 2013-2015 PUC-Rio/Laboratorio TeleMidia
 
 This file is part of NCLua.
 
@@ -60,7 +60,7 @@ along with NCLua.  If not, see <http://www.gnu.org/licenses/>.  */
    if (unlikely (!(cond)))                                      \
      {                                                          \
       fprintf (stderr, "%s:%d: ASSERTION FAILED!\n--> %s\n",    \
-                 __FILE__, __LINE__, STRINGIFY(cond));          \
+               __FILE__, __LINE__, STRINGIFY (cond));           \
       abort ();                                                 \
      }                                                          \
   }                                                             \
@@ -109,7 +109,7 @@ ATTR_PRINTF_FORMAT (1, 2)
 /*-
  * Outputs a numbered entry separator.
  */
-static ATTR_UNUSED inline void
+static ATTR_UNUSED void
 TRACE_SEP (void)
 {
   static int n = 1;
