@@ -1,4 +1,4 @@
---[[ Copyright (C) 2013-2014 PUC-Rio/Laboratorio TeleMidia
+--[[ Copyright (C) 2013-2015 PUC-Rio/Laboratorio TeleMidia
 
 This file is part of NCLua.
 
@@ -34,9 +34,23 @@ local table = table
 local tostring = tostring
 local type = type
 
+---
+-- Fail-safe require.
+--
+local function optrequire (name)
+   local t = {pcall (require, name)}
+   if t[1] == false then
+      return nil, t[2]
+   else
+      return table.unpack (t, 2)
+   end
+end
+tests.optrequire = optrequire
+
 local tests0 = require ('tests0')
 local canvas = require ('nclua.canvas')
-local socket = require ('nclua.event.socket')
+local socket = optrequire ('nclua.event.tcp_socket')
+local soup = optrequire ('nclua.event.http_soup')
 local stopwatch = require ('nclua.event.stopwatch')
 _ENV = nil
 
@@ -198,7 +212,7 @@ end
 
 function tests.dump (...)
    local args = {...}
-   local n = tests.maxi (args)
+   local n = tests.maxi (args) or 0
    for i=1,n do
       dump (args[i])
       cat ('\n')
@@ -267,6 +281,29 @@ function tests.proj (i, ...)
    return args[i]
 end
 
+---
+-- Returns N if N is in the interval [LOWER,UPPER].
+-- If LOWER is given and N < LOWER, returns LOWER.
+-- If UPPER is given and N > UPPER, returns UPPER.
+--
+function tests.range (lower, n, upper)
+   if lower and n < lower then
+      return lower
+   elseif upper and n > upper then
+      return upper
+   else
+      return n
+   end
+end
+
+---
+-- Delay for a specified amount of time.
+--
+do
+   tests.sleep = function (s) tests.usleep (s * 10^6) end
+   tests.usleep = tests0.usleep
+end
+
 ---
 -- Outputs arguments to stdout prefixed with a time-stamp.
 --
@@ -289,36 +326,55 @@ function tests.trace_sep ()
    tests._trace_sep_number = tests._trace_sep_number + 1
 end
 
+
+---------------------------- File manipulation  ----------------------------
+
 ---
--- Writes string STR to file FILE.
+-- Returns the filename part of PATH.
 --
-function tests.write_file (s, file)
-   local file = assert (io.open (file, 'wb'))
-   assert (file:write (str))
-   file:close ()
+function tests.basename (path)
+   return path:match ('^.*/([^/]*)$') or path
 end
 
 ---
--- Delay for a specified amount of time.
+-- Returns the directory part of PATH.
+--
+function tests.dirname (path)
+   return path:match ('^(.*)/[^/]*$') or '.'
+end
+
+---
+-- Concatenate the given arguments to make a path.
+--
+function tests.mkpath (...)
+   return table.concat ({...}, '/')
+end
+
+---
+-- Wrapper to os.tmpname; it workarounds bogus names returned by
+-- os.tmpname() on MinGW.
+--
+function tests.tmpname ()
+   local name = os.tmpname ()
+   if tests.is_windows () then
+      return name:gsub ('\\', 'xslashx')
+   else
+      return name
+   end
+end
+
+---
+-- Returns true if PATH points to a directory.
 --
 do
-   tests.sleep = function (s) tests.usleep (s * 10^6) end
-   tests.usleep = tests0.usleep
+   tests.dir_exists = tests0.dir_exists
 end
 
 ---
--- Returns N if N is in the interval [LOWER,UPPER].
--- If LOWER is given and N < LOWER, returns LOWER.
--- If UPPER is given and N > UPPER, returns UPPER.
+-- Returns true if PATH points to a regular file.
 --
-function tests.range (lower, n, upper)
-   if lower and n < lower then
-      return lower
-   elseif upper and n > upper then
-      return upper
-   else
-      return n
-   end
+do
+   tests.file_exists = tests0.file_exists
 end
 
 ---
@@ -338,6 +394,15 @@ function tests.read_file (file)
    return result
 end
 
+---
+-- Writes string STR to file FILE.
+--
+function tests.write_file (s, file)
+   local file = assert (io.open (file, 'wb'))
+   assert (file:write (str))
+   file:close ()
+end
+
 
 -------------------------- Random data generators --------------------------
 
@@ -395,7 +460,7 @@ end
 -- longer needed.
 --
 local function rand_file (f_rand_string, size)
-   local tmp = os.tmpname ()
+   local tmp = tests.tmpname ()
    local file = assert (io.open (tmp, 'wb'))
    assert (file:write (f_rand_string (size)))
    file:close ()
@@ -556,7 +621,10 @@ end
 ---------------------------------- CAIRO -----------------------------------
 
 do
-   tests.cairo_version = tests0.cairo_version
+   tests.cairo_check_version = tests0.cairo_check_version
+   tests.cairo_get_version = tests0.cairo_get_version
+   tests.CAIRO_MAJOR, tests.CAIRO_MINOR, tests.CAIRO_MICRO
+      = tests.cairo_get_version ()
 end
 
 
@@ -604,9 +672,22 @@ end
 --
 -- If EPSILON is given, admit difference in EPSILON * 100% of the pixels.
 --
+local REF_CAIRO_DIRS = {
+   ('ref-cairo-%d-%d'):format (tests.CAIRO_MAJOR, tests.CAIRO_MINOR),
+   'ref-cairo-any',
+}
 function tests.canvas.check_ref (canvas, serial, epsilon)
-   local name = debug.getinfo (2).short_src
-   local path = name:gsub ('%.lua', '-'..serial..'-ref.png')
+   local s = debug.getinfo (2).short_src
+   local root = tests.dirname (s)
+   local base = tests.basename (s):gsub ('%.lua', '-'..serial..'-ref.png')
+   local path = nil
+   for _, dir in ipairs (REF_CAIRO_DIRS) do
+      path = tests.mkpath (root, dir, base)
+      if tests.file_exists (path) then
+         break
+      end
+   end
+   assert (path, path)
    local ref = assert (canvas:new (path), path)
    return tests.canvas.surface_equals (canvas:_surface (),
                                        ref:_surface (),
@@ -617,8 +698,17 @@ end
 -- Dumps a new reference picture with the given serial.
 --
 function tests.canvas.dump_ref (canvas, serial)
-   local name = debug.getinfo (2).short_src
-   local path = name:gsub ('%.lua', '-'..serial..'-ref.png')
+   local s = debug.getinfo (2).short_src
+   local root = tests.dirname (s)
+   local base = tests.basename (s):gsub ('%.lua', '-'..serial..'-ref.png')
+   local path = nil
+   for _, dir in ipairs (REF_CAIRO_DIRS) do
+      if tests.dir_exists (dir) then
+         path = tests.mkpath (root, dir, base)
+         break
+      end
+   end
+   assert (path, path)
    tests.trace (('*** DUMPING REF: %s ***'):format (path))
    return assert (canvas:_dump_to_file (path))
 end
@@ -676,7 +766,7 @@ end
 --
 function tests.server.start (server)
    server.pid = nil
-   server.pidfile = os.tmpname ()
+   server.pidfile = tests.tmpname ()
    local str = ('sh %s/server.sh %s --verbose --pid="%s" --port=%d %s')
       :format (tests.mk.srcdir,
                tests.mk.srcdir,
@@ -716,4 +806,21 @@ function tests.socket.cycle_until (func)
    until func ()
 end
 
+
+----------------------------------  SOUP  ----------------------------------
+
+do
+   tests.soup = {}
+end
+
+---
+-- Calls soup.cycle () until function FUNC returns true.
+--
+function tests.soup.cycle_until (func)
+   repeat
+      soup.cycle ()
+      tests.usleep (10^4)       -- 10ms
+   until func ()
+end
+
 return tests
